/*
 * $Id: attrcmng.c,v 1.5 2002/05/06 09:59:43 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Handle attract scene(title, gameover, pause).
 *
 * @version $Revision: 1.5 $
 */
#include <sys/bios.h>
#include <stdlib.h>

#include "putipati.h"
#include "attrcmng.h"
#include "gamemng.h"
#include "prefmng.h"
#include "soundmng.h"
#include "stagemng.h"
#include "stagedta.h"
#include "ball.h"
#include "man.h"
#include "frag.h"

static Ranking ranking;

/* Load ranking. */
Boolean loadPrefs() {
  return loadPreferences((char far *)&ranking, sizeof(ranking));
}

/* Save ranking. */
Boolean savePrefs() {
  return savePreferences((char far *)&ranking, sizeof(ranking));
}

static char rankChar[4][2] = {
  {'s', 't'}, {'n', 'd'}, {'r','d'}, {'t', 'h'}
};

static void drawRank(int n, int x, int y) {
  int c;
  drawNum2(n+1, x, y-2);
  if ( n < 3 ) c = n;
  else c = 3;
  drawAlp2(rankChar[c][0], x, y-1);
  drawAlp2(rankChar[c][1], x, y);
}

static int htCnt;

/* Initialize title screen. */
void gotoTitle() {
  int x, y, c, i;
  gameStatus.status = TITLE;
  clearDisplay();
  clearMen();
  clearFrags();
  initBalls();
  initStage();
  c = TITLE_CHAR;
  for ( x=1 ; x<17 ; x++ ) {
    for ( y=19 ; y<23 ; y++ ) {
      screen_fill_char(SCREEN2, y, x, 1, 1, c);
      c++;
    }
  }
  for ( i=0 ; i<6 ; i++ ) {
    addBall(i%3, 1, 
	    randN(SCREEN_TEXT_HEIGHT-1)<<11, randN(SCREEN_TEXT_WIDTH-10)<<11, 
	    randNS(180), randNS(180));
  }
  y = 16;
  for ( i=0 ; i<10 ; i++, y-- ) {
    drawRank(i, y, 5);
    drawNum2(ranking.score[i], y, 14);
  }
  htCnt = -1;
}

static char startChar[] = {
  'p', 'u', 's', 'h', ' ', 'a', 'b', ' ', 'b', 'u', 't', 't', 'o', 'n'
};

/* Handle title screen. */
void handleTitle() {
  int key = key_press_check();
  moveBalls();
  htCnt++;
  if ( (htCnt&63) == 0 ) {
    int x, i = 0;
    for ( x=2 ; x<16 ; x++, i++ ) {
      drawAlp2(startChar[i], 3, x);
    }
  } else if ( (htCnt&63) == 32 ) {
    screen_fill_char(SCREEN2, 3, 2, 1, 14, 0);
  }

  if ( !(key & KEY_START) ) {
    startPrsd = FALSE;
  } else if ( !startPrsd ) { 
    startPrsd = TRUE;
    loopf = FALSE;
    return;
  }
  if ( !(key & (KEY_A | KEY_B)) ) {
    aPrsd = FALSE;
  } else if ( !aPrsd ) { 
    aPrsd = TRUE;
    clearBalls();
    startGame();
  }
}

static int hgCnt;
static int rank;

/* Initialize gameover. */
void gotoGameover() {
  int i, j;
  gameStatus.status = GAMEOVER;
  initStage();
  clearDisplay();
  hgCnt = 0;
  rank = -1;
  for ( i=0 ; i<10 ; i++ ) {
    if ( score > ranking.score[i] ) {
      for ( j=9 ; j>i ; j-- ) {
	ranking.score[j] = ranking.score[j-1];
      }
      ranking.score[i] = score;
      rank = i;
      savePrefs();
      break;
    }
  }
}

static char overChar[] = {
  'g', 'a', 'm', 'e', 'o', 'v', 'e', 'r'
};
static char cmpChar[] = {
  'c', 'o', 'm', 'p', 'l', 'e', 't', 'e', 'd'
};
static char ysChar[] = {
  'y', 'o', 'u', 'r', ' ', 's', 'c', 'o', 'r', 'e', ' ', 'i', 's'
};
static char yrChar[] = {
  'r', 'a', 'n', 'k'
};

/* Handle gameover. */
void handleGameover() {
  int key = key_press_check();
  int x, i = 0;
  switch ( hgCnt ) {
  case 0:
    if ( stage < STAGE_NUM ) {
      for ( x=5 ; x<13 ; x++, i++ ) {
	drawAlp2(overChar[i], 18, x);
      }
    } else {
      for ( x=4 ; x<13 ; x++, i++ ) {
	drawAlp2(cmpChar[i], 18, x);
      }
    }
    break;
  case 32:
    for ( x=2 ; x<15 ; x++, i++ ) {
      drawAlp2(ysChar[i], 14, x);
    }
    break;
  case 64:
    drawNum2(score, 11, 10);
    if ( rank == -1 ) {
      hgCnt = 128;
    }
    break;
  case 96:
    for ( x=7 ; x<11 ; x++, i++ ) {
      drawAlp2(yrChar[i], 6, x);
    }
    break;
  case 128:
    drawRank(rank, 4, 10);
    break;
  }
  hgCnt++;
  moveMen(key);

  if ( !(key & (KEY_A | KEY_B)) ) {
    aPrsd = FALSE;
  } else if ( !aPrsd ) { 
    aPrsd = TRUE;
    hgCnt = ((hgCnt-1)&0xffe0) + 32;
    if ( hgCnt > 128 ) {
      gotoTitle();
    }
  }
  if ( hgCnt > 256 ) {
    gotoTitle();
  }
}

/* Initialize ranking score. */
void initRanking() {
  int i;
  for ( i=0 ; i<RANKING_NUM ; i++ ) {
    ranking.score[i] = 1000*(10-i);
  }
}

static char pauseChar[] = {
  'p', 'a', 'u', 's', 'e'
};

/* Initialize game pause. */
void gotoPause() {
  int x, i=0;
  gameStatus.status = PAUSE;
  screen_fill_char(SCREEN2, 11, 4, 5, 9, 0);
  screen_fill_char(SCREEN2, 15, 4, 1, 1, BLOCK_CHAR+5);
  screen_fill_char(SCREEN2, 15, 5, 1, 7, BLOCK_CHAR+1);
  screen_fill_char(SCREEN2, 15, 12, 1, 1, BLOCK_CHAR+6);
  screen_fill_char(SCREEN2, 12, 12, 3, 1, BLOCK_CHAR+2);
  screen_fill_char(SCREEN2, 11, 12, 1, 1, BLOCK_CHAR+3);
  screen_fill_char(SCREEN2, 11, 5, 1, 7, BLOCK_CHAR+1);
  screen_fill_char(SCREEN2, 11, 4, 1, 1, BLOCK_CHAR+4);
  screen_fill_char(SCREEN2, 12, 4, 3, 1, BLOCK_CHAR+2);
  for ( x=6 ; x<11 ; x++, i++ ) {
    drawAlp2(pauseChar[i], 13, x);
  }
}

/* Handle game pause. */
void handlePause() {
  int key = key_press_check();

  if ( !(key & KEY_START) ) {
    startPrsd = FALSE;
  } else if ( !startPrsd ) { 
    startPrsd = TRUE;
    clearBalls();
    stopBgm();
    gotoTitle();
    return;
  }
  if ( !(key & (KEY_A | KEY_B)) ) {
    aPrsd = FALSE;
  } else if ( !aPrsd ) { 
    int x, y;
    aPrsd = TRUE;
    gameStatus.status = IN_GAME;
    screen_fill_char(SCREEN2, 11, 4, 5, 9, 0);
    for ( y=11 ; y<=15 ; y++ ) {
      for ( x=4 ; x<=12 ; x++ ) {
	setSpotChar(x, y);
      }
    }
  }
}


/* Messages in game(stage, time). */
static char stgMsg[] = {'s', 't', 'a', 'g', 'e', 0, 0};
static int ppsmc;

void startStageMessage(int n) {
  screen_fill_char(SCREEN1, 0, 0, 1, SCREEN_TEXT_HEIGHT, 0);
  if ( n >= 10 ) { 
    stgMsg[5] = 1;
    stgMsg[6] = n%10;
  } else {
    stgMsg[5] = 0;
    stgMsg[6] = n;
  }
  ppsmc = -1;
}

void printStageMessage(int c) {
  int x, i, ox;
  if ( ppsmc == c ) return;
  ppsmc = c;
  if ( c < 256 ) {
    ox = (16*c)>>8;
  } else {
    ox = 16;
  }
  screen_fill_char(SCREEN1, 0, 0, 1, SCREEN_TEXT_HEIGHT, 0);
  x = ox;
  for ( i=0 ; i<5 ; i++, x+=ox ) {
    screen_fill_char(SCREEN1, 0, x>>3, 1, 1, ALP_CHAR+(stgMsg[i]-'a'));
  }
  for ( ; i<7 ; i++, x+=ox ) {
    screen_fill_char(SCREEN1, 0, x>>3, 1, 1, NUM_CHAR+stgMsg[i]);
  }
}

static int pptmc, tgDec, tgDecOfs, gx;

void startTimeMessage(int st) {
  screen_fill_char(SCREEN1, 0, 0, 1, SCREEN_TEXT_HEIGHT, 0);
  screen_fill_char(SCREEN1, 0, 1, 1, 6, ARROW_CHAR);
  screen_fill_char(SCREEN1, 0, 11, 1, 6, ARROW_CHAR+1);
  pptmc = -1;
  tgDecOfs = st/6; tgDec = st-tgDecOfs;
  gx = 0;
}

int timerBlink = 1;

void printTimeMessage(int t) {
  int ct, tc;
  ct = t/GAME_TIME_INTERVAL + 1;
  if ( t < tgDec ) {
    screen_fill_char(SCREEN1, 0, 1+gx, 1, 1, 0);
    screen_fill_char(SCREEN1, 0, 16-gx, 1, 1, 0);
    gx++; tgDec -= tgDecOfs;
  }
  switch ( gx ) {
  case 4:
    tc = t&31;
    if ( tc == 0 ) {
      timerBlink = 1;
      screen_fill_char(SCREEN1, 0, 8, 1, 1, NUM_CHAR+(ct/10));
      screen_fill_char(SCREEN1, 0, 9, 1, 1, NUM_CHAR+(ct%10));
      playSe(WARN_SE_NUM);
    } else if ( tc < 16 ) {
      if ( pptmc == ct ) return;
      pptmc = ct;
      screen_fill_char(SCREEN1, 0, 8, 1, 1, NUM_CHAR+(ct/10));
      screen_fill_char(SCREEN1, 0, 9, 1, 1, NUM_CHAR+(ct%10));
    } else if ( tc == 16 ) {
      timerBlink = 0;
      screen_fill_char(SCREEN1, 0, 8, 1, 2, 0);
      timerBlink = 0;
      playSe(WARN_SE_NUM);
    }
    break;
  case 5:
    tc = t&15;
    if ( tc == 0 ) {
      timerBlink = 1;
      screen_fill_char(SCREEN1, 0, 1+gx, 1, 1, ARROW_CHAR);
      screen_fill_char(SCREEN1, 0, 16-gx, 1, 1, ARROW_CHAR+1);
      screen_fill_char(SCREEN1, 0, 8, 1, 1, NUM_CHAR+(ct/10));
      screen_fill_char(SCREEN1, 0, 9, 1, 1, NUM_CHAR+(ct%10));
      playSe(WARN_SE_NUM);
    } else if ( tc < 8 ) {
      if ( pptmc == ct ) return;
      pptmc = ct;
      screen_fill_char(SCREEN1, 0, 8, 1, 1, NUM_CHAR+(ct/10));
      screen_fill_char(SCREEN1, 0, 9, 1, 1, NUM_CHAR+(ct%10));
    } else if ( tc == 8 ) {
      timerBlink = 0;
      screen_fill_char(SCREEN1, 0, 1+gx, 1, 1, 0);
      screen_fill_char(SCREEN1, 0, 16-gx, 1, 1, 0);
      screen_fill_char(SCREEN1, 0, 8, 1, 2, 0);
      timerBlink = 0;
      playSe(WARN_SE_NUM);
    }
    break;
  default:
    timerBlink = 1;
    if ( pptmc == ct ) return;
    pptmc = ct;
    screen_fill_char(SCREEN1, 0, 8, 1, 1, NUM_CHAR+(ct/10));
    screen_fill_char(SCREEN1, 0, 9, 1, 1, NUM_CHAR+(ct%10));
    break;
  }
}

static char clrMsg[] = {'b', 'o', 'n', 'u', 's'};

void startClearMessage(int t) {
  int clearBonus = 0, i;
  screen_fill_char(SCREEN1, 0, 0, 1, SCREEN_TEXT_HEIGHT, 0);
  if ( t > 0 ) {
    clearBonus = (t/GAME_TIME_INTERVAL + 1)*100;
    score += clearBonus;
    drawScore();
  }
  for ( i=0 ; i<5 ; i++ ) {
    screen_fill_char(SCREEN1, 0, i+1, 1, 1, ALP_CHAR+(clrMsg[i]-'a'));
  }
  drawNum(clearBonus, 0, 16);
}
